@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002, 2003, 2004, 2005, 2006 Christophe Kalt
+** Copyright (C) 2002-2008 Christophe Kalt
 **
 ** This file is part of shmux,
 ** see the LICENSE file for details on your rights.
@@ -29,7 +29,7 @@
 #include "target.h"
 #include "term.h"
 
-static char const rcsid[] = "@(#)$Id: loop.c,v 1.56 2006/08/10 23:49:05 kalt Exp $";
+static char const rcsid[] = "@(#)$Id: loop.c 1538 2008-12-18 16:13:19Z kalt $";
 
 extern char *myname;
 
@@ -277,7 +277,8 @@ struct child *kid;
 			    {
 			      str = (char *) malloc(strlen(*left)
 						    + strlen(start) + 1);
-			      sprintf(str, "%s%s", *left, start);
+			      snprintf(str, strlen(*left) + strlen(start) + 1,
+					"%s%s", *left, start);
 			    }
 
 			  if (analyzer_lnrun(analyzer,
@@ -403,9 +404,10 @@ struct child *kid;
 		    
 		    old = *left;
 		    *left = (char *) malloc(strlen(start) + leftlen + 1);
-		    strcpy(*left, old);
+		    strlcpy(*left, old, strlen(start) + leftlen + 1);
 		    free(old);
-		    strcpy((*left) + leftlen, start);
+		    strlcpy((*left) + leftlen, start,
+			sizeof((*left) + leftlen));
 		  }
 	      }
 	  }
@@ -912,7 +914,11 @@ u_int ctimeout, utest;
 	/* Check (or not) for input */
 	pfd[0].fd = tty_fd();
 	if (pfd[0].fd >= 0)
+#if !defined(BROKEN_POLL)
 	    pfd[0].events = POLLIN;
+#else
+            pfd[0].events = 0;
+#endif
 	else
 	  {
 	    pfd[0].events = 0;
@@ -928,6 +934,13 @@ u_int ctimeout, utest;
 	    spawn_mode = SPAWN_FATAL;
 	    break;
 	  }
+#if defined(BROKEN_POLL)
+        if (pfd[0].fd >= 0)
+          {
+            pfd[0].revents = POLLIN;
+            pollrc += 1;
+          }
+#endif
 
 	/* Abort? */
 	switch (got_sigint)
@@ -979,12 +992,13 @@ u_int ctimeout, utest;
 		  }
 
 		/* Something is going on.. */
-		dprint("idx=%d[%s] fd=%d(%d) IN=%d OUT=%d ERR=%d HUP=%d",
+		dprint("idx=%d[%s] fd=%d(%d) IN=%d OUT=%d ERR=%d HUP=%d (%X)",
 		       idx, what, pfd[idx].fd, idx%3,
 		       (pfd[idx].revents & POLLIN) != 0,
 		       (pfd[idx].revents & POLLOUT) != 0,
 		       (pfd[idx].revents & POLLERR) != 0,
-		       (pfd[idx].revents & POLLHUP) != 0);
+		       (pfd[idx].revents & POLLHUP) != 0,
+                       pfd[idx].revents);
 
 		if (idx % 3 != 0 || idx == 0)
 		  {
@@ -1011,6 +1025,7 @@ u_int ctimeout, utest;
 		      {
 			if (idx == 0)
 			  {
+#if !defined(BROKEN_POLL)
 			    if (sz == 0)
 				eprint("Unexpected empty read(/dev/tty) result");
 			    else
@@ -1018,6 +1033,13 @@ u_int ctimeout, utest;
 				       strerror(errno));
 			    if (sz == 0 || errno != EINTR)
 				tty_restore();
+#else
+			    if (sz != 0)
+				eprint("Unexpected read(/dev/tty) error for: %s",
+				       strerror(errno));
+			    if (errno != EINTR)
+				tty_restore();
+#endif
 			  }
 			else
 			  {
@@ -1508,7 +1530,8 @@ u_int ctimeout, utest;
 			if (fd >= 0)
 			  {
 			    char buf[4];
-			    sprintf(buf, "%u", WEXITSTATUS(status));
+			    snprintf(buf, sizeof(buf), "%u",
+				WEXITSTATUS(status));
 			    write(fd, buf, strlen(buf));
 			    close(fd);
 			    free(fn);
