@@ -1,10 +1,10 @@
 /*
-** Copyright (C) 2002, 2003 Christophe Kalt
+** Copyright (C) 2002-2008 Christophe Kalt
 **
 ** This file is part of shmux,
 ** see the LICENSE file for details on your rights.
 **
-** $Id: os.h,v 1.4 2003/11/08 01:17:27 kalt Exp $
+** $Id: os.h 1435 2008-04-03 01:43:18Z kalt $
 */
 
 #if !defined(_OS_H_)
@@ -29,6 +29,10 @@
 
 #include <string.h>
 
+#if !defined(HAVE_STRLCPY)
+# define strlcpy(x, y, z) strcpy(x, y)
+#endif
+
 /* shmux exit codes */
 #define RC_OK	 0
 #define RC_QUIT	 1
